package hpdos_rdma_offloaded;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetSocketAddress;
import java.util.Properties;

import com.ibm.darpc.DaRPCServerEndpoint;
import com.ibm.darpc.DaRPCServerGroup;
import com.ibm.disni.RdmaServerEndpoint;

import hpdos_rdma_offloaded.handler.NetworkHandler;
import hpdos_rdma_offloaded.handler.RegistrationHandler;
import hpdos_rdma_offloaded.handler.ReplicationHandler;
import hpdos_rdma_offloaded.handler.StorageHandler;
import hpdos_rdma_offloaded.rdma_protocol.Request;
import hpdos_rdma_offloaded.rdma_protocol.Response;
import hpdos_rdma_offloaded.service.MetadataServerService;

public class MetadataServer implements Runnable{
    private String host;
    private int poolsize;
    private int recvQueue;
    private int sendQueue;
    private int wqSize = recvQueue;
    private boolean polling;
    private int maxinline = 0;
    private int rdma_port;

    boolean isMaster;
    StorageHandler storageHandler;
    NetworkHandler networkHandler;
    MetadataServerService metadataServerService;
    RegistrationHandler registrationHandler = null;
    public final static Properties properties= new Properties();;

    public MetadataServer(Properties properties) throws IOException {
        this.isMaster = Boolean.valueOf((String)properties.get("app.MASTER"));
        this.storageHandler = new StorageHandler(properties);
        this.networkHandler = new NetworkHandler(this.storageHandler, properties);
        this.metadataServerService =  new MetadataServerService(this.networkHandler);
        // if (this.isMaster) {
            this.registrationHandler = new RegistrationHandler(properties);
        // }
    }
    
    
    public void run()
    {
        host = (String) properties.get("app.HOST");
        poolsize = Integer.valueOf( (String)properties.get("app.cpu_affinity"));
        recvQueue = Integer.valueOf( (String)properties.get("app.rdma_receive_queue"));
        sendQueue = Integer.valueOf( (String) properties.get("app.rdma_send_queue"));
        wqSize = recvQueue;
        maxinline = Integer.valueOf((String) properties.get("app.rdma_max_inline"));
        rdma_port = Integer.valueOf((String) properties.get("app.rdma_server_port"));
        polling = Boolean.valueOf((String) properties.get("app.rdma_polling"));
        long[] clusterAffinities = new long[poolsize];
        for (int i = 0; i < poolsize; i++){
            long cpu = 1L << i;
            clusterAffinities[i] = cpu;
        }
        // MetadataServerService rpcService = new MetadataServerService();
        DaRPCServerGroup<Request, Response> group = null;
        try {
            group = DaRPCServerGroup.createServerGroup(this.metadataServerService, clusterAffinities, -1, maxinline, polling, recvQueue, sendQueue, wqSize, 32);
        } catch (Exception e) {
            e.printStackTrace();
        };
        RdmaServerEndpoint<DaRPCServerEndpoint<Request, Response>> serverEp = null;
        try {
            serverEp = group.createServerEndpoint();
        } catch (IOException e) {
            e.printStackTrace();
        }
        InetSocketAddress address = new InetSocketAddress(host, rdma_port);
        try {
            serverEp.bind(address, 100);
        } catch (Exception e) {
            e.printStackTrace();
        }
        while(true){
            try {
                // System.out.println("Listening to RDMA requests a, IP: " + host + " , PORT: " + rdma_port);
                serverEp.accept();
                // System.out.println("Accepted connection.");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void main(String[] args) throws IOException, InterruptedException {
        InputStream inputStream = new FileInputStream(args[0]);
        properties.load(inputStream);
        boolean isMaster = Boolean.valueOf((String)properties.get("app.MASTER")); 
        if (isMaster) {
            System.out.println("STARTING MASTER SERVICES.");
        } else {
            System.out.println("STARTING FOLLOWER SERVICES.");
        }
        Runnable t1 = new MetadataServer(properties);
        Thread t = new Thread(t1);
        
        t.start();
        
    }
}
